#include "file.bi"
#include "datetime.bi"

const version as string = "0.3"
/'
Nexuizstats 0.3
Analyses Nexuiz 2.5.2 event log files and calculates a player ranking.
Copyright (C) 2016  Martin Wohlauer (7Saturn) <mwohlauer@yahoo.de>, <7Saturn@gmx.de>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

The results are exported into a html table
Last Change: 2016-04-05-04-21
'/



'############## Some neccessary definitions ##############



#ifdef __FB_LINUX__
	#define PATHSEPARATOR "/"
	#define ALLFILES "*"
	#define OSTYPE "Linux"
#endif
#ifdef __FB_openBSD__
	#define PATHSEPARATOR "/"
	#define ALLFILES "*"
	#define OSTYPE "OpenBSD"
#endif
#ifdef __FB_DARWIN__ 
	#define PATHSEPARATOR "/"
	#define ALLFILES "*"
	#define OSTYPE "Unix"
#endif
#ifdef __FB_DOS__
	#define PATHSEPARATOR "\"
	#define ALLFILES "*.*"
	#define ISDOS 1
	#define OSTYPE "DOS"
#endif
#ifndef __FB_DOS__
	#define ISDOS 0
#endif
#ifdef __FB_WIN32__
	#define PATHSEPARATOR "\"
	#define ALLFILES "*.*"
	#define OSTYPE "Windows"
#endif
#ifdef __FB_64BIT__
	#define BITVERSION "64"
#else
	#ifdef __FB_DOS__
		#define BITVERSION "16"
	#else
		#define BITVERSION "32"
	#endif
#endif


type table_with_id
	id as uinteger 'The current id of a player with a certain nick
	nick as string 'Nickname of a palyer
	Kills as uinteger 'Frags done by the player
	Deaths as uinteger 'Times fragged by other player
	Suicides as uinteger 'killed himself
	Accidents as uinteger 'Times the player got killed by mapitems or falling damage
	teamkills as uinteger 'Teamkills done by the player
	isbot as uinteger 'Is this player a bot?
	ctfsteal as uinteger 'Times the player took the enemies flag from their flag spot
	ctfdrop as uinteger 'Times the player dropped the flag (usually means fragged by enemy)
	ctfpickup as uinteger 'Times the player took the flag laying around somewhere else, than the flag spot
	ctfcapture as uinteger 'Times successfully took the enemies flag to the own flag spot
	ctfreturn as uinteger 'Times the player returned the own flag to the own flag spot
	lasttimeactive as integer 'Time-Stamp/-difference, when the player has had his last appearence on the server
end type

type table_without_id
	nick as string 'Nickname of a palyer
	Kills as uinteger 'Frags done by the player
	Deaths as uinteger 'Times fragged by other player
	Suicides as uinteger 'killed himself
	Accidents as uinteger 'Times the player got killed by mapitems or falling damage
	teamkills as uinteger 'Teamkills done by the player
	isbot as uinteger 'Is this player a bot?
	ctfsteal as uinteger 'Times the player took the enemies flag from their flag spot
	ctfdrop as uinteger 'Times the player dropped the flag (usually means fragged by enemy)
	ctfpickup as uinteger 'Times the player took the flag laying around somewhere else, than the flag spot
	ctfcapture as uinteger 'Times successfully took the enemies flag to the own flag spot
	ctfreturn as uinteger 'Times the player returned the own flag to the own flag spot
	ratio as double 'kills/deaths
	efficiency as double 'kills/(kills+deaths)
	alldeaths as uinteger 'deaths+accidents+suicides
	allratio as double 'ratio including accidents and suicides
	allefficency as double 'efficency including accidents and suicides
	ctfratio as double 'ctfcapture/ctfsteals
	lasttimeactive as integer 'Timestamp/-difference, when the player has had his last appearence on the server
end type



'############## Functions and subs used ##############



declare function direxists(byval tochangeto as string) as uinteger
declare sub getfilenames(byval tosearch as string, returnarray() as string)
declare function numberoffiles(byval tosearch as string) as integer
declare function htmlonlytext(byval inputstring as string) as string
declare function replace(ByVal original as string, ByVal replacement as string, ByVal position as integer, ByVal toremove as integer) as string
declare function trailzeros(byval inputnumber as string, zeros as integer) as string
declare function marginless overload (byval a as string) as string
declare function round(i as double,digits as uinteger,trailing as uinteger) as string
declare function nickcolored(byval inputstring as string) as string
declare sub addemptyentrytotable overload (byval nickname as string, byval isbot as uinteger, byval id as uinteger, tablearray() as table_with_id)
declare sub addemptyentrytotable overload (byval nickname as string, byval isbot as uinteger, tablearray() as table_without_id)
declare function getnickpositionfromtable overload (byval nickname as string, tablearray() as table_with_id) as uinteger
declare function getnickpositionfromtable(byval nickname as string, tablearray() as table_without_id) as uinteger
declare function getnickfromfiletable(byval id as uinteger, tablearray() as table_with_id) as string
declare function getpositionbyid (byval id as uinteger,tablearray() as table_with_id) as uinteger
declare function getpositionbynick (byval nick as string, tablearray() as table_without_id) as uinteger
declare function numberofentriesintable overload (tablearray() as table_with_id) as uinteger
declare function numberofentriesintable overload (tablearray() as table_without_id) as uinteger
declare function isinarray(stringarray() as string, byval searchstring as string) as uinteger
declare sub readsavedstats(byval savedstatsfile as string, filenamelist() as string, statsarray() as table_without_id)
declare sub savestats(byval savedstatsfile as string, filenamelist() as string, statsarray() as table_without_id, byval timestamp as integer)
declare function csvstring (byval inputstring as string) as string
declare function insert(byval text as string, byval toinsert as string, atplace as integer) as string
declare function xmlonlytext(byval inputstring as string) as string
declare sub autowrapprint overload (byval printstring as string, break as uinteger)
declare sub autowrapprint (byval printstring as string)

function direxists(byval tochangeto as string) as uinteger
	'Checks, whether the directory with the given name is accessible.
	dim startpath as string = curdir
	if chdir(tochangeto) = 0 then
		chdir startpath
		return 1
	else
		return 0
	end if
end function

sub getfilenames(byval tosearch as string, returnarray() as string)
	'saves the names of files the, matching the pattern given by
	'»tosearch« into the returnarray
	'Last change: 2016-02-13-23-44
	if tosearch="" then tosearch=ALLFILES
	dim filename as string
	dim listlength as uinteger=numberoffiles(tosearch) 'Number of files
	dim counter as uinteger=0
	redim returnarray(listlength) as string
	filename = dir(tosearch, (&h21 and (not &h10)))
	do while len(filename)
		counter=counter+1
		returnarray(counter)=filename
		filename = dir( )
	loop
end sub

function numberoffiles(byval tosearch as string) as integer
	'Returns the number files, matching the pattern given by »tosearch«
	'Last Change: 2016-02-13-23-30
	dim filename as string
	dim numberprinted as uinteger = 0 'Number of found files
	if tosearch="" then tosearch=ALLFILES
	filename = dir(tosearch, (&h21 and (not &h10)))
	do while len(filename)
		numberprinted=numberprinted+1
		filename = dir( )
	loop
	return numberprinted
end function

function htmlonlytext(byval inputstring as string) as string
	'Replaces non-standard and HTML-control-characters of a string with
	'the HTML-Code-replacements/-entities.
	'Requires replace-function.
	'Last change: 2016-02-08-16-01
	dim counter as uinteger
	dim length as uinteger = len(inputstring)
	if length=0 then return ""
	counter = 0
	while counter < length
		counter = counter +1
		if mid(inputstring,counter,1) = chr(38) then inputstring=replace(inputstring,"&amp;",counter-1,1)
		if mid(inputstring,counter,1) = chr(62) then inputstring=replace(inputstring,"&gt;",counter-1,1)
		if mid(inputstring,counter,1) = chr(60) then inputstring=replace(inputstring,"&lt;",counter-1,1)
		if (asc(mid(inputstring,counter,1))> 126) or (asc(mid(inputstring,counter,1)) < 32) then inputstring=replace(inputstring, ("&#"&asc(mid(inputstring,counter,1))&";"), counter-1,1)
		length=len(inputstring)
	wend
	return inputstring
end function

function replace(ByVal original as string, ByVal replacement as string, ByVal position as integer, ByVal toremove as integer) as string
	'Replaces »toremove« characters after character no. »position«
	'within »original« with the string »replacement«.
	'Last change: 2015-12-05-20-11
	if original = "" then return replacement
	if position < 0 then position = 0
	if toremove < 0 then toremove = 0
	if position >= len(original) then return original+replacement
	if toremove+position > len(original) then toremove=len(original)-position
	return left(original,position)+replacement+right(original,len(original)-position-toremove)
end function

function trailzeros(byval inputnumber as string, zeros as integer) as string
	'Adds trailing zeros or removes excess digits after the point to/
	'from a number containing string
	'requires function marginless
	'Last change: 2016-02-10-17-16
	if zeros<0 then zeros=0
	dim dotposition as integer
	dim counter as uinteger
	inputnumber=marginless(inputnumber)
	if inputnumber="" then inputnumber="0"
	dotposition = instr(inputnumber,".")
	if dotposition=0 then
		if zeros=0 then
			return inputnumber
		else
			inputnumber=inputnumber+"."
			for counter = 1 to zeros
				inputnumber=inputnumber+"0"
			next counter
			return inputnumber
		end if
	else
		if zeros = 0 then
			return left(inputnumber,dotposition-1)
		end if
		if (dotposition = len(inputnumber)) and (zeros = 0) then
			return left(inputnumber,len(inputnumber)-1)
		end if
		if (zeros-(len(inputnumber)-dotposition)) = 0 then
			return inputnumber
		end if
		if (zeros-(len(inputnumber)-dotposition)) < 0 then
			inputnumber=left(inputnumber,dotposition+zeros)
			return inputnumber
		end if
		if (zeros-(len(inputnumber)-dotposition)) > 0 then
			for counter = 1 to zeros-(len(inputnumber)-dotposition)
				inputnumber=inputnumber+"0"
			next counter
			return inputnumber
		end if
		return inputnumber
	end if
end function

function marginless(byval a as string) as string
	'Removes spaces on the left and right side of string »a«.
	return ltrim(rtrim(a))
end function

function round(i as double,digits as uinteger,trailing as uinteger) as string
	'Rounds a given number to digits number of decimals after the point
	'if trailing is set to 1, then trailing zeros are added, to always
	'get the exact number of digits adter the point
	'Last change: 2016-02-10-16-36
	dim counter as uinteger
	dim textvalue as string
	dim dotposition as uinteger
	'just in case someone was so smart to devide by zero bevor using this
	if str(i)="inf" then return "inf"
	if str(i)="-nan" then return "-nan"
	'Round here
	textvalue=str(cint(i*10^digits)/10^digits)
	dotposition=instr(textvalue,".")
	if dotposition = 0 then
		textvalue=textvalue+"."
		dotposition=instr(textvalue,".")
	end if
	'Check, whether crappy basics double-format kicks your butt
	if len(textvalue)-dotposition > digits then
		textvalue=left(textvalue,dotposition+digits)
		if right(textvalue,1)="9" then
			textvalue=str(i+1/10^(digits+1))
			dotposition=instr(textvalue,".")
			textvalue=left(textvalue,dotposition+digits)
			return textvalue
		end if
		if right(textvalue,1)="1" then
			textvalue=left(textvalue,dotposition+digits)
			dotposition=instr(textvalue,".")
		end if
	end if
	'Add zeros, if requested and possible
	if (len(textvalue)-dotposition < digits) and (trailing = 1) then
		while len(textvalue)-dotposition < digits
			textvalue=textvalue+"0"
		wend
	end if
	if digits=0 and right(textvalue,1)="." then textvalue=left(textvalue,len(textvalue)-1)
	return textvalue
end function

function nickcolored(byval inputstring as string) as string
	'Replaces Nexuiz string color-codes of the sort "^?" with
	'corresponding html/css color tags
	'Last change: 2016-03-17-00-49
	dim position as uinteger
	dim code as string
	dim openbracket as uinteger = 0
	dim customcolor as string
	while instr(inputstring,"^")
		position=instr(inputstring,"^")
		if not position = len(inputstring) then
			code = mid(inputstring,position,2)
			select case code
				case "^0"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #454545;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #454545;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^1"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #ff0000;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #ff0000;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^2"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #00ff00;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #00ff00;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^3"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #ffff00;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #ffff00;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^4"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #0000ff;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #0000ff;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^5"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #00ffff;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #00ffff;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^6"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #ff00ff;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #ff00ff;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^7"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #ffffff;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #ffffff;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^8"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #505050;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #505050;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^9"
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #808080;"&chr(34)&">",position-1,2)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #808080;"&chr(34)&">",position-1,2)
						openbracket=1
					end if
				case "^x"
					customcolor=mid(inputstring,position+2,3)
					
					if openbracket=1 then
						inputstring=replace(inputstring,"</span><span style="&chr(34)&"color: #"&left(customcolor,1)&"0"&mid(customcolor,2,1)&"0"&right(customcolor,1)&"0;"&chr(34)&">",position-1,5)
					else
						inputstring=replace(inputstring,"<span style="&chr(34)&"color: #"&left(customcolor,1)&"0"&mid(customcolor,2,1)&"0"&right(customcolor,1)&"0;"&chr(34)&">",position-1,5)
						openbracket=1
					end if

				case else
					inputstring=replace(inputstring,"",position-1,2)
			end select
			if openbracket=1 then inputstring=inputstring+"</span>"
		end if
	wend
	return inputstring
end function

sub addemptyentrytotable(byval nickname as string, byval isbot as uinteger, byval id as uinteger, tablearray() as table_with_id)
	'Adds a new player to the stats list, if there doesn't already exist
	'one with that nick. If the a player with that nick already exists,
	'it will do nothing.
	dim tablecounter as uinteger = 0
	dim isthere as uinteger = 0
	'Let's see, if we can find it
	for tablecounter = 1 to ubound(tablearray)
		if tablearray(tablecounter).id=id then isthere=1:exit for
	next tablecounter
	'If it's already there, nothing to be done.
	if isthere then exit sub
	'OK, it's not there, so we'll create it
	'Get the next position available
	isthere = getnickpositionfromtable("", tablearray())
	'But first, let's make sure, there's enough space for it:
	if isthere = 0 then
		'Apparently there's no entry with no set nickname, so this thing is full! Let's expand it.
		redim preserve tablearray(ubound(tablearray)*2)
		isthere = getnickpositionfromtable("", tablearray())
	end if
	tablearray(isthere).id=id
	tablearray(isthere).nick=nickname
	tablearray(isthere).isbot=isbot
end sub

sub addemptyentrytotable overload (byval nickname as string, byval isbot as uinteger, tablearray() as table_without_id)
	'Adds a new player to the stats list, if there doesn't already exist
	'one with that nick. If the a player with that nick already exists,
	'it will do nothing.
	dim tablecounter as uinteger = 0
	dim isthere as uinteger = 0
	'Let's see, if we can find it
	for tablecounter = 1 to ubound(tablearray)
		if tablearray(tablecounter).nick=nickname then isthere=1:exit for
	next tablecounter
	'If it's already there, nothing to be done.
	if isthere then exit sub
	'OK, it's not there, so we'll create it
	'Get the next position available
	isthere = getnickpositionfromtable("", tablearray())
	'But first, let's make sure, there's enough space for it:
	if isthere = 0 then
		'Apparently there's no entry with no set nickname, so this thing is full! Let's expand it.
		redim preserve tablearray(ubound(tablearray)*2)
		isthere = getnickpositionfromtable("", tablearray())
	end if
	tablearray(isthere).nick=nickname
	tablearray(isthere).isbot=isbot
end sub

function getnickpositionfromtable overload (byval nickname as string, tablearray() as table_with_id) as uinteger
	'Returns the position of a player inside the array, that uses the
	'given nickname. If not already included, it returns zero.
	dim tablecounter as uinteger
	for tablecounter = 1 to ubound(tablearray)
		if tablearray(tablecounter).nick=nickname then return tablecounter
	next tablecounter
	'We went through the entiry array but didn't find the searched nick.
	'It's simply not there. So we'll return 0.
	return 0
end function

function getnickpositionfromtable(byval nickname as string, tablearray() as table_without_id) as uinteger
	'Returns the position of a player inside the array, that uses the
	'given nickname. If not already included, it returns zero.
	dim tablecounter as uinteger
	for tablecounter = 1 to ubound(tablearray)
		if tablearray(tablecounter).nick=nickname then return tablecounter
	next tablecounter
	'We went through the entiry array but didn't find the searched nick.
	'It's simply not there. So we'll return 0.
	return 0
end function

function getpositionbyid (byval id as uinteger,tablearray() as table_with_id) as uinteger
	'Returns the position of a player inside a files stats array by
	'looking for the mentioned id
	dim tablecounter as uinteger
	for tablecounter = 1 to ubound(tablearray)
		if tablearray(tablecounter).id=id then return tablecounter
	next tablecounter
	return 0
end function

function numberofentriesintable overload (tablearray() as table_with_id) as uinteger
	'Returns the actual number of entries used in the given stats table array
	dim number as uinteger=0
	while tablearray(number+1).nick <> ""
		number=number+1
		'Just to be on the safe side for that one situation, when the entry-list is exactly full
		if number=ubound(tablearray) then exit while
	wend
	return number
end function

function numberofentriesintable overload (tablearray() as table_without_id) as uinteger
	'Returns the actual number of entries used in the given stats table array
	dim number as uinteger=0
	while tablearray(number+1).nick <> ""
		number=number+1
		'Just to be on the safe side for that one situation, when the entry-list is exactly full
		if number=ubound(tablearray) then exit while
	wend
	return number
end function

function extractdate (byval inputstring as string) as string
	'Extracts the date-part from a string like this:
	'YYYY-MM-DD HH:MM:SS
	'and transformes it into a string like this:
	'MM-DD-YYYY
	'This way, it's easier to calculate with freebasic's build in functions
	dim yearpart as string
	dim monthpart as string
	dim daypart as string
	if len(inputstring) < 10 then return ""
	inputstring=left(inputstring,10)
	yearpart=left(inputstring,4)
	monthpart=mid(inputstring,6,2)
	daypart=right(inputstring,2)
	if DATEVALUE(monthpart+"-"+daypart+"-"+yearpart) = 0 then
		return ""
	else
		return (monthpart+"-"+daypart+"-"+yearpart)
	end if
end function

function isinarray(stringarray() as string, byval searchstring as string) as uinteger
	'Checks, whether a certain string is included in a string array
	'(but not as a substring inside of a string of that array)
	dim arraycounter as integer
	for arraycounter = lbound(stringarray) to ubound(stringarray)
		if stringarray(arraycounter)=searchstring then return 1
	next arraycounter
	return 0
end function

sub readsavedstats(byval savedstatsfile as string, filenamelist() as string, statsarray() as table_without_id)
	'Reads previously saved statistical values, to continue analysis with new files added to it
	const statsfileheader as string = "Nexuiz stats save file, revision 1"
	dim availablefile as uinteger = freefile
	dim errorflag as uinteger
	dim counter as uinteger
	dim inputstring as string
	dim inputnumber as integer
	dim timestamp as integer
	errorflag=open (savedstatsfile for input as availablefile)
	if errorflag <> 0 then
		print "Couldn't open ";chr(34);savedstatsfile;chr(34);" for input. No preceding data can be used."
		exit sub
	end if
	line input #availablefile,inputstring
	'Some simple checking, wether this actually has been saved with nexuiz stats.
	if (inputstring <> statsfileheader) then
		print "File ";chr(34);savedstatsfile;chr(34);" is no valid stats save file, aborting."
		errorflag=3
		system(errorflag)
	end if
	'Some lines will simply be skipped, as they are just for a more
	'human readable format
	input #availablefile,inputstring
	input #availablefile,inputstring
	input #availablefile,timestamp
	input #availablefile,inputstring
	'Reading number of filenames listet in the following lines
	input #availablefile,inputnumber
	redim filenamelist(inputnumber)
	input #availablefile,inputstring
	'Reading file names
	for counter = 1 to inputnumber
		input #availablefile,filenamelist(counter)
	next counter
	input #availablefile,inputstring
	'reading number of players saved in the following lines
	input #availablefile,inputnumber
	redim statsarray(inputnumber)
	'Reading players stats so far
	for counter = 1 to inputnumber
		'»line« is important, as nicks can include any kind of characters
		line input #availablefile,statsarray(counter).nick
		input #availablefile,statsarray(counter).Kills
		input #availablefile,statsarray(counter).Deaths
		input #availablefile,statsarray(counter).Suicides
		input #availablefile,statsarray(counter).Accidents
		input #availablefile,statsarray(counter).teamkills
		input #availablefile,statsarray(counter).isbot
		input #availablefile,statsarray(counter).ctfsteal
		input #availablefile,statsarray(counter).ctfdrop
		input #availablefile,statsarray(counter).ctfpickup
		input #availablefile,statsarray(counter).ctfcapture
		input #availablefile,statsarray(counter).ctfreturn
		input #availablefile,statsarray(counter).lasttimeactive
		statsarray(counter).lasttimeactive=timestamp-statsarray(counter).lasttimeactive
	next counter
	close availablefile
end sub

sub savestats(byval savedstatsfile as string, filenamelist() as string, statsarray() as table_without_id, byval timestamp as integer)
	'This part saves collected statistical data. It includes the file
	'names of the files already analysed, so they won't be read again.
	'It includes *all* data collectable from the log files, even if not
	'exported to HTML or other format file. This way, one can decide to
	'change, whether ctf-data shall be included or the format be HTML
	'anyways at a later time, without having to read all of it again.
	dim availablefile as uinteger = freefile
	dim errorflag as uinteger
	dim counter as uinteger
	errorflag=open (savedstatsfile for output as availablefile)
	if errorflag <> 0 then
		print "Couldn't open ";chr(34);savedstatsfile;chr(34);" for output, stats data was *not* saved into the requested file. Aborting."
		errorflag=4
		system(errorflag)
	end if
	'Actual saving of information
	print #availablefile,"Nexuiz stats save file, revision 1"
	print #availablefile,"Do not mess with this file!"
	print #availablefile,"Timestamp:"
	print #availablefile,timestamp
	print #availablefile,"Number of read log files:"
	print #availablefile,ubound(filenamelist)
	print #availablefile,"Already read files:"
	for counter = 1 to ubound(filenamelist)
		print #availablefile,filenamelist(counter)
	next counter
	print #availablefile,"Number of found players:"
	print #availablefile,numberofentriesintable(statsarray())
	for counter = 1 to numberofentriesintable(statsarray())
		'The values to be calculated are not included. They will be re-
		'calculated anyways, so there's no point in using up disk space
		'for them.
		print #availablefile,statsarray(counter).nick
		print #availablefile,statsarray(counter).Kills
		print #availablefile,statsarray(counter).Deaths
		print #availablefile,statsarray(counter).Suicides
		print #availablefile,statsarray(counter).Accidents
		print #availablefile,statsarray(counter).teamkills
		print #availablefile,statsarray(counter).isbot
		print #availablefile,statsarray(counter).ctfsteal
		print #availablefile,statsarray(counter).ctfdrop
		print #availablefile,statsarray(counter).ctfpickup
		print #availablefile,statsarray(counter).ctfcapture
		print #availablefile,statsarray(counter).ctfreturn
		print #availablefile,statsarray(counter).lasttimeactive
	next counter
	close availablefile
end sub

function csvstring (byval inputstring as string) as string
	'Converts strings into csv compliant notation, meaning doubling
	'quotation marks inside the string and add quotation marks to the
	'beginning and end of string
	dim counter as uinteger = 0
	if inputstring="" then return ""
	while counter < len(inputstring)
		counter = counter + 1
		if mid(inputstring,counter,1)=chr(34) then
			inputstring=insert(inputstring,chr(34),counter)
			counter = counter+1
		end if
	wend
	return chr(34)+inputstring+chr(34)
end function

function insert(byval text as string, byval toinsert as string, atplace as integer) as string
	'Inserts string »toinsert« into string »text« after position
	'»atplace«.
	if toinsert = "" then return text
	if text="" then return toinsert
	if atplace <= 0 then return toinsert+text
	if atplace > len(text) then return text+toinsert
	return left(text,atplace)+toinsert+right(text,len(text)-atplace)
end function

function xmlonlytext(byval inputstring as string) as string
	'Replaces non-standard and HTML-control-characters of a string with
	'the XML-Code-replacements/-entities for ISO-8859-1 or nothing, if not defined
	'requires replace-function
	'Last change: 2016-04-01-07-34
	dim counter as uinteger
	dim length as uinteger = len(inputstring)
	if length=0 then return ""
	counter = 0
	while counter < length
		counter = counter +1
		if mid(inputstring,counter,1) = chr(38) then inputstring=replace(inputstring,"&amp;",counter-1,1)
		if mid(inputstring,counter,1) = chr(62) then inputstring=replace(inputstring,"&gt;",counter-1,1)
		if mid(inputstring,counter,1) = chr(60) then inputstring=replace(inputstring,"&lt;",counter-1,1)
		if (((asc(mid(inputstring,counter,1))> 126) and (asc(mid(inputstring,counter,1)) < 160)) or (asc(mid(inputstring,counter,1)) < 32)) then inputstring=replace(inputstring, "", counter-1,1):counter=counter-1
		length=len(inputstring)
	wend
	return inputstring
end function

sub autowrapprint (byval printstring as string)
	'To make the default break = 1 (as in ordinary print without »;«
	'easier to write
	'Last change: 2016-04-01-03-11
	autowrapprint (printstring, 1)
end sub

sub autowrapprint (byval printstring as string, break as uinteger)
	'Prints a given string from the current position, but automatically
	'wraps, if the string ist to long for the current line. If break is
	'set to > 0 (default, see overloaded function), then the last
	'printed line will be concluded with a carriage return.
	'Last change: 2016-04-01-17-17
	if printstring="" then
		if break = 0 then
			exit sub
		else
			print
		end if
	end if
	printstring=printstring+" "
	dim maxwidth as integer = 81-pos()
	dim startposition as integer = 1
	dim endposition as integer=0
	dim spacesearch as integer=0
	while not ((endposition = len(printstring)) or (startposition > len(printstring)) or (inkey=chr(27)))
		maxwidth = 81-pos()
		spacesearch=startposition
		while instr(spacesearch+1,printstring," ")-startposition < maxwidth and instr(spacesearch+1,printstring," ") <> 0
			spacesearch=instr(spacesearch+1,printstring," ")
		wend
		if instr(spacesearch+1,printstring," ")-spacesearch > 80 then spacesearch=spacesearch+79
		if spacesearch=startposition then spacesearch=spacesearch+80
		endposition=spacesearch-1
		if endposition = 0 then endposition = len(printstring)
		if endposition-startposition+1 > maxwidth then endposition=startposition+maxwidth-1
		print mid(printstring,startposition,endposition-startposition+1);
		startposition=endposition+2
		if startposition > len(printstring) then
			if break > 0 then
				print
				exit sub
			end if
		else
			print
		end if	
	wend
end sub



'############## Variable declarations ##############



dim help as uinteger = 0 'Flag, if help shall be printed
dim errorflag as uinteger = 0 'Flag, whether an error in command line parameters occured
dim startpath as string = curdir() 'remember, where we started from, to be able to get back there, if necessary
dim logdir as string = "" 'directory with the log files, can be set via console parameters
dim defaultoutfile as string = "nexuiz.html" 'Not const, as it can be changed later on by command line parameters
dim outfile as string 'Output file name, can be overriden by commandline parameters
dim outputfileformat as uinteger 'Format-Type of the output file, 0=HTML, 1=plain text, 2=CSV, 3=XML
dim savefile as string 'The filename of a stats save file
dim makestats as uinteger = 0 'Shall a stats save file be used or not? 0=no, 1=yes

const defaultfilefilter as string = "nexuiz????????.log" 'The standard search pattern for log files in the logdir
dim filefilter as string 'Search pattern for log files in the logdir, can be overridden by console parameter
dim nobots as uinteger = 1 'Shall bots be included, or not? Can be overriden by console parameter.
dim ctf as uinteger = 0 'Shall ctf stats be included?
dim datefilter as uinteger = 0 'Days, above which a player is not logged in the stats, any more (30, 90 or 365 (days) or 0 for all stats
dim currenttime as integer = 0 'The current day, for calculating time since last playing

dim instring as string 'The lines read from inputfiles
dim id as uinteger 'players id
dim idposition as uinteger 'position-counter for id search loops
dim newplayername as string 'nickname read from file
dim isbot as uinteger 'is the current player a bot or not? 0=no, 1=yes

dim eventtype as string 'What kind of event is logged in that line? kill, nick change, etc....
dim killtype as string 'what kind of kill is it?

dim attackerid as integer 'The id from the attacker, reported in the log
dim victimid as uinteger 'The id from the victim, reported in the log

dim counter as uinteger 'counter for file-loop
dim counter2 as uinteger 'counter for various for loops
dim changed1 as uinteger 'counter for sort part
dim changed2 as uinteger 'counter for sort part
dim newfilescounter as uinteger 'Counts newly added files in case of stats file used
dim linecounter as uinteger 'for debugging purposes: Which line of the current input file has been read?

dim filelist() as string 'List of log file names
dim knownfiles() as string 'List of already known files (to be read from saved stats status)

'These /have/ to be redims, not dims, as they have to be redimable later as well. Otherwise they will be static arrays which can't be changed in dimensions
redim allstats(64) as table_without_id 'This is, where the stats from all log files go to
redim filestats(64) as table_with_id 'This is, where the stats from all log files will go



'############## Command line parameter parsing and help ##############



const allowedparameters = "--logdir --outname --filefilter --withbots --text --help --ctf --onemonth --onequarter --oneyear --statssave --csv --xml "
const allowedtwopartedparameters = "--logdir --outname --filefilter --statssave " 'These need also be included in the allowed parameters!

print "Nexuizstats " & version & " Copyright (C) 2016  Martin Wohlauer"
autowrapprint("This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under the terms of the GPLv3 license.")

'Short parsing check
if command(1)="" then
	print "No parameters given, using defaults on all values."
else
	if not command(1) = "" and instr(allowedparameters,command(1)+" ") = 0 then
		print "Error in command line parameters. Unknown parameter ";chr(34);command(1);chr(34);"."
		help = 1
		errorflag = 1 
	end if
	for counter = 2 to __FB_ARGC__ - 1
		if (instr(allowedparameters,command(counter)+" ") = 0) and instr(allowedtwopartedparameters,command(counter-1)+" ") = 0 then
			print "Error in command line parameters. Unknown parameter ";chr(34);command(counter);chr(34);"."
			help = 1
			errorflag = 1
		end if
	next counter

	'See, if someone needs instructions
	for counter = 0 to __FB_ARGC__ - 1
		if command(counter) = "--help" then
			help = 1
			exit for
		end if
	next counter

	for counter = 0 to __FB_ARGC__ - 1
		if command(counter) = "--withbots" then nobots = 0
		if command(counter) = "--ctf" then ctf = 1
		if command(counter) = "--text" then outputfileformat = 1
		if command(counter) = "--csv" then outputfileformat = 2
		if command(counter) = "--xml" then outputfileformat = 3
		if command(counter) = "--onemonth" then datefilter = 31
		if command(counter) = "--onequarter" then datefilter = 91
		if command(counter) = "--oneyear" then datefilter = 366
	next counter
end if

if help = 1 then
	print
	print "Usuage:"
	autowrapprint(command(0)+" [--logdir <statsfilename>] [--statssave <statssavefile>] [--outname <outputfilename>] [--text] [--csv] [--xml] [--filefilter <searchpatter>] [--withbots] [--help]")
	print
	print "File name related switches:"
	print
	print " --logdir     Overrides the default directory (current) with the provided one."
	print " --filefilter Overrides the default search pattern ";chr(34);"nexuiz????????.log";chr(34);"."
	print "              Keep in mind, that there will always be 8 digits used for the"
	print "              numbering."
	print " --outname    Overrides the default filename ";chr(34);"nexuiz.html";chr(34);", ";chr(34);"nexuiz.txt";chr(34);","
	print "              ";chr(34);"nexuiz.csv";chr(34);" or ";chr(34);"nexuiz.xml";chr(34);" with the provided one. You can enter"
	print "              a complete file path there, or just a filename. Relative paths are";
	print "              allowed. An already existing file with that name will be"
	print "              overwritten."
	print " --statssave  Reads already existing statistical values, including the filenames";
	print "              of already read log files (if it exists) and later saves the"
	print "              updated values and filenames into the given file. This is highly"
	print "              recommended, when used on a regular basis with greater amounts of"
	print "              stats log files (> 100) to minimize usuage of system ressources."
	print
	print "The default output format is HTML but can be overridden by the following"
	print "switches:"
	print
	print " --text       The outputfile will be a simple text file."
	print " --csv        The outputfile will not be a CSV table."
	print " --xml        The outputfile will be a XML text file."
	print
	print "Filtering players by time of last activity is possible with these switches:"
	print
	print " --onemonth   Only players active during the last 30 days will be included."
	print " --onequarter Only players active during the last 90 days will be included."
	print " --oneyear    Only players active during the last 365 days will be included."
	print
	print "Other options:"
	print
	print " --ctf        Set this switch, if you want stats on CTF-events included as well."
	print " --withbots   Set this switch, if you want events involving bots to be included"
	print "              into the stats."
	print " --help       Displays this help. If used, all other options will be neglected."
	print
	print "Overriding switches:"
	print
	autowrapprint("The switches --text, --csv, --xml, --onemonth, --onequarter and --oneyear override previous switches setting a time filter or output format. So for example, when using --text after --xml, the format will be set to text, overriding the former --xml switch.")
	print
	print "Errorflags:"
	print " 0 Everything went well."
	print " 1 Console parameters were faulty."
	print " 2 The directory path provided to search in for the logs, can't be accessed/"
	print "   doesn't exist."
	print " 3 The provided stats save file has the wrong format."
	print " 4 The stats save file could not be written."
	print " 5 One of the event log files in the log folder could not be opened for"
	print "   reading."
	print " 6 The output file could not be written."
	system (errorflag)
end if

if nobots = 0 then
	print "--withbots switch found. Bots will be included."
else
	print "Bots will not be included."
end if
if ctf = 1 then
	print "--ctf switch found. CTF stats will be included."
else
	print "CTF stats will not be included."
end if
select case outputfileformat
	case 0
		print "No format switch found. Outputfile will be a HTML file."
	case 1
		print "--text switch found. Outputfile will be a text file."
		defaultoutfile="nexuiz.txt"
	case 2
		print "--csv switch found. Outputfile will be a CSV table."
		defaultoutfile="nexuiz.csv"
	case 3
		print "--xml switch found. Outputfile will be a XML format text file."
		defaultoutfile="nexuiz.xml"
	case else
		'Shouldn't be happening
end select

select case datefilter
	case 0
		print "No time filter switch uses. Players of all time will be included."
	case 31
		print "--onemonth switch found. Including only players active during the last 30 days."
	case 91
		print "--onequarter switch found. Including only players active during the last 90"
		print "days."
	case 366
		print "--oneyear switch found. Including only players active during the last 365 days."
end select

'Get the involved filenames, if provided

'did the user provide an input directory? If so, let's use it.
for counter = 0 to __FB_ARGC__ - 1
	if command(counter) = "--logdir" then
		print "--logdir switch found."
		logdir=command(counter+1)
		if logdir="" or logdir = "--statssave" or logdir = "--outname" or logdir = "--text" or logdir = "--csv" or logdir = "--xml" or logdir = "--withbots" or logdir = "--filefilter" or logdir = "--ctf" or logdir = "--onemonth" or logdir = "--onequarter" or logdir = "--oneyear" then
			print"But missing path."
			print "Run ";chr(34);command(0);" --help";chr(34);" to view usuage hints."
			errorflag=1
			system(errorflag)
		else
			print "Directorypath taken from parameters: ";chr(34);logdir;chr(34)
		end if
	end if
next counter

'If not, take the current one
if logdir = "" then
	logdir = curdir
	print "Automatic choice for input directory: ";logdir
end if

'Did the user provide an output file name? If so, let's use it.
for counter = 0 to __FB_ARGC__ - 1
	if command(counter) = "--outname" then
		print "--outname switch found."
		outfile=command(counter+1)
		if outfile="" or outfile = "--statssave" or outfile = "--withbots" or outfile = "--text" or outfile = "--csv" or outfile = "--xml" or outfile = "--filefilter" or outfile = "--logdir" or outfile = "--ctf" or outfile = "--onemonth" or outfile = "--onequarter" or outfile = "--oneyear" then
			print"But missing filename."
			print "Run ";chr(34);command(0);" --help";chr(34);" to view usuage hints."
			errorflag=1
			system(errorflag)
		else
			print "Filename taken from parameters: ";chr(34);outfile;chr(34)
		end if
	end if
next counter

'If not, take the default for the (selected) format
if outfile = "" then
	outfile=defaultoutfile
	print "Using default filename ";outfile;" for output."
end if

'Did the user provide a stats save file? If so, let's use it.
for counter = 0 to __FB_ARGC__ - 1
	if command(counter) = "--statssave" then
		print "--statssave switch found."
		savefile=command(counter+1)
		if savefile="" or savefile = "--outname" or savefile = "--withbots" or savefile = "--text" or savefile = "--csv" or savefile = "--xml" or savefile = "--filefilter" or savefile = "--logdir" or savefile = "--ctf" or savefile = "--onemonth" or savefile = "--onequarter" or savefile = "--oneyear" then
			print"But missing filename."
			print "Run ";chr(34);command(0);" --help";chr(34);" to view usuage hints."
			errorflag=1
			system(errorflag)
		else
			print "Filename taken from parameters: ";chr(34);savefile;chr(34)
			makestats=1
		end if
	end if
next counter

'Someone has set a new wild card filter? If so, let's use it.
for counter = 0 to __FB_ARGC__ - 1
	if command(counter) = "--filefilter" then
		print "--filefilter switch found."
		filefilter=command(counter+1)
		if filefilter="" or filefilter = "--withbots" or filefilter = "--statssave" or filefilter = "--text" or filefilter = "--csv" or filefilter = "--xml" or filefilter = "--outname" or filefilter = "--logdir" or filefilter = "--ctf" or filefilter = "--onemonth" or filefilter = "--onequarter" or filefilter = "--oneyear" then
			print"But missing filter pattern."
			print "Run ";chr(34);command(0);" --help";chr(34);" to view usuage hints."
			errorflag=1
			system(errorflag)
		else
			print "Filename pattern taken from parameters: ";chr(34);filefilter;chr(34)
		end if
		
	end if
next counter

'Otherwise, just take the standard filter pattern
if filefilter = "" then
	filefilter=defaultfilefilter
	print "No filename pattern specified, using default ";chr(34);filefilter;chr(34);"."
end if



'############## Some prework ##############


'Read a stats save file, if requested
if makestats = 1 then readsavedstats(savefile, knownfiles(), allstats())

'switch to the directory with the logs
if direxists(logdir) then
	chdir logdir
else
	print "Path ";chr(34);logdir;chr(34);" not found/accessible, aborting!"
	errorflag=2
	system (errorflag)
end if

'lets retreive those file names
getfilenames(filefilter,filelist())

'let's find out, how many of them we have
dim numberoflogfiles as integer=ubound(filelist)

'And if there are none, just get the hell out of here.
if numberoflogfiles<1 then autowrapprint("No log files found. Did you use the proper directory path and wild card filter? Use --help switch to get further instructions."):errorflag=0:system (errorflag)



'############## Here the actual file analysis takes place ##############



'now let's take one file at a time:
for counter = 1 to numberoflogfiles
	if (isinarray(knownfiles(),filelist(counter)) <> 1) or (makestats = 0) then
		if makestats = 1 then newfilescounter=newfilescounter+1
		errorflag=open(filelist(counter) for input as #1)
		if errorflag <> 0 then
			print "Couldn't open log file ";chr(34);filelist(counter);chr(34);" for reading. Aborting!"
			errorflag=5
			system (errorflag)
		end if
		line input #1, instring
		linecounter=1
		' Format check, whether it actually is a Nexuiz stats log file
		if instring = ":logversion:3" then
			'It's OK, seems to be a valid Nexuiz stats log file
			'Let's delete the old data from the last file (if there ever was one)
			redim filestats(64)
			while not eof(1)
				line input #1,instring
				linecounter=linecounter+1
				'skip that useless ":" in the first column
				instring=right(instring,len(instring)-1)
				
				'is it a useful line?
				eventtype=left(instring,instr(instring,":"))
				select case eventtype
					case "join:"
						'So it's a joining player.
						'We won't need the prefix "join:" any more:
						instring=right(instring,len(instring)-5)
						
						'the id is the part before the next ":"
						id=val(left(instring,instr(instring,":")-1))
						'Let's cut that id away
						instring=right(instring,len(instring)-instr(instring,":"))
						'And the slot as well, we won't need it
						instring=right(instring,len(instring)-instr(instring,":"))
						
						'Next part identifies whether it's a bot, or not
						if left(instring,instr(instring,":")-1)="bot" then isbot=1 else isbot=0
						'Now, that we know that, let's cut that away, as well.
						instring=right(instring,len(instring)-instr(instring,":"))
						
						'The rest will be the nickname
						newplayername=instring
						
						'We now have all the required data.
						'But actually that guy doesn't have to be a new player for that round/file. Maybe he just reconnected but kept his nick.
						'If just reconnected, nothing will happen, here, as the nick is already in the list:
						addemptyentrytotable(newplayername,isbot,id,filestats())

						'So now we either have an old entry with new id, or a new entry with, of course, new id.
						'In any case, we'll have to update the new id for the player with this nick
						idposition=getnickpositionfromtable(newplayername,filestats())
						filestats(idposition).id=id
						
						'Now just one tiny little check: Has this name already been used by a bot or is the newly entering player a bot, using an already existing name?
						if (filestats(idposition).isbot = 0) and (isbot = 1) then filestats(idposition).isbot = 1
						
						filestats(idposition).lasttimeactive=currenttime
						'That's all to be done here. Next line...
					case "name:"
						'It's a nick change
						'We won't need the prefix "name:" any more:
						instring=right(instring,len(instring)-5)
						
						'First part is the id
						id=val(left(instring,instr(instring,":")-1))
						'Let's cut that id away
						instring=right(instring,len(instring)-instr(instring,":"))
						
						'The rest will be the nickname
						instring=right(instring,len(instring)-instr(instring,":"))
						newplayername=instring
						
						'Now let's give him a new nick:
						filestats(getpositionbyid(id,filestats())).nick=newplayername
						filestats(idposition).lasttimeactive=currenttime

						'That's all to be done here. Next line...
					case "kill:"
						'Someone got killed
						'We will not need that "kill:"-part any more.
						instring=right(instring,len(instring)-instr(instring,":"))

						'Assess Type of kill (can be suicide or something...)
						killtype=left(instring,instr(instring,":")-1)
						'Cut away killtype:
						instring=right(instring,len(instring)-instr(instring,":"))
						
						'Let's get the attacker's id
						attackerid=val(left(instring,instr(instring,":")-1))
						'Cut away attacker's id:
						instring=right(instring,len(instring)-instr(instring,":"))

						'Let's get the victim's id,
						victimid=val(left(instring,instr(instring,":")-1))
						'Cut away victim's id:
						instring=right(instring,len(instring)-instr(instring,":"))

						'Assessing, what exactly happened:
						select case killtype
							case "accident"
								'That's what the map can do to you... Let's add it to the player
								'victim and attacker are the same, but let's takte victim, just to be on the safe side)
								filestats(getpositionbyid(victimid,filestats())).Accidents=filestats(getpositionbyid(victimid,filestats())).Accidents+1
								filestats(getpositionbyid(victimid,filestats())).lasttimeactive=currenttime
							case "suicide"
								'This is, when the player screws himself
								filestats(getpositionbyid(victimid,filestats())).suicides=filestats(getpositionbyid(victimid,filestats())).suicides+1
								filestats(getpositionbyid(victimid,filestats())).lasttimeactive=currenttime
							case "tk"
								'OK, someone screwed a teammate, so it's actually a Teamkill
								'Only do this, if no bot involced, or bots allowed
								if (filestats(getpositionbyid(attackerid,filestats())).isbot+filestats(getpositionbyid(victimid,filestats())).isbot = 0) or nobots=0 then
									'One Teamkill for the attacker:
									filestats(getpositionbyid(attackerid,filestats())).teamkills=filestats(getpositionbyid(attackerid,filestats())).teamkills+1
									filestats(getpositionbyid(attackerid,filestats())).lasttimeactive=currenttime
									'And one Death for the Victim:
									filestats(getpositionbyid(victimid,filestats())).deaths=filestats(getpositionbyid(victimid,filestats())).deaths+1
									filestats(getpositionbyid(victimid,filestats())).lasttimeactive=currenttime
								end if
							case else
								'No other kind of death, so it has to be a simple frag.
								'Only do this, if no bot involved, or bots allowed
								if (filestats(getpositionbyid(attackerid,filestats())).isbot+filestats(getpositionbyid(victimid,filestats())).isbot = 0) or nobots=0 then
									'Add kill to the attacker:
									filestats(getpositionbyid(attackerid,filestats())).kills=filestats(getpositionbyid(attackerid,filestats())).kills+1
									filestats(getpositionbyid(attackerid,filestats())).lasttimeactive=currenttime
									'And add death to the victim:
									filestats(getpositionbyid(victimid,filestats())).deaths=filestats(getpositionbyid(victimid,filestats())).deaths+1
									filestats(getpositionbyid(victimid,filestats())).lasttimeactive=currenttime
								end if
						end select
					case "ctf:"
						'This only happens, when the game is CTF and someone did something with the flag.
						'We will not need that "ctf:"-part any more.
						instring=right(instring,len(instring)-instr(instring,":"))

						'Assess Type of ctf-event (can be anything from stealing to returning...)
						killtype=left(instring,instr(instring,":")-1)
						'Cut away ctf-event
						instring=right(instring,len(instring)-instr(instring,":"))
						'And the flag color as well, not important
						instring=right(instring,len(instring)-instr(instring,":"))
						'Let's get the player's id, it's all that's left
						id=val(instring)

						'We now got everything we need
						'Let's assess what exactly happened:
						idposition=getpositionbyid(id,filestats())
						filestats(idposition).lasttimeactive=currenttime
						select case killtype
							case "steal"
								filestats(idposition).ctfsteal=filestats(idposition).ctfsteal+1
							case "dropped"
								filestats(idposition).ctfdrop=filestats(idposition).ctfdrop+1
							case "pickup"
								filestats(idposition).ctfpickup=filestats(idposition).ctfpickup+1
							case "capture"
								filestats(idposition).ctfcapture=filestats(idposition).ctfcapture+1
							case "return"
								filestats(idposition).ctfreturn=filestats(idposition).ctfreturn+1
							case "returned"
								'Nothing to be done, in this case
							case else
								'This shouldn't be happening, except for malformed log files
								 autowrapprint("This is odd. I don't know this CTF event... (event " & chr(34) & killtype & chr(34)&"file "&chr(34)&filelist(counter)&chr(34)&", Line "&str(linecounter+1)&"). Are you using some kind of mod? Please inform the author of this program, so that he may add it, somehow.")
						end select
					case "time:"
						'We will not need that "time:"-part any more.
						instring=right(instring,len(instring)-instr(instring,":"))
						'this will set the time stamp to the last entry, that had a time change in it
						'as there will be a time stamp before or after every event, it's the closest thing to the actual last action of player
						currenttime=datevalue(extractdate(instring))
					case else 
				end select
			wend
			close 1
			'Now we have all the data collected from this one log file.
			'Let's integrate the new data into the big list from all files:
			for counter2 = 1 to numberofentriesintable(filestats())
				idposition=getnickpositionfromtable(filestats(counter2).nick,allstats())
				if idposition = 0 then
					'OK, this is the very first entry of this player in the whole list
					addemptyentrytotable(filestats(counter2).nick,filestats(counter2).isbot,allstats())
					idposition=getnickpositionfromtable(filestats(counter2).nick,allstats())
				end if
				'We (now) know that fellow, so let's just add the values into his account
				allstats(idposition).Kills=allstats(idposition).Kills+filestats(counter2).kills
				allstats(idposition).Deaths=allstats(idposition).Deaths+filestats(counter2).deaths
				allstats(idposition).Suicides=allstats(idposition).Suicides+filestats(counter2).suicides
				allstats(idposition).Accidents=allstats(idposition).Accidents+filestats(counter2).Accidents
				allstats(idposition).teamkills=allstats(idposition).teamkills+filestats(counter2).teamkills
				'With being a bot or not, that's a bit more complex
				if (allstats(idposition).isbot = 0) and (filestats(counter2).isbot = 1) then
					allstats(idposition).isbot=1
				end if
				allstats(idposition).ctfsteal=allstats(idposition).ctfsteal+filestats(counter2).ctfsteal
				allstats(idposition).ctfdrop=allstats(idposition).ctfdrop+filestats(counter2).ctfdrop
				allstats(idposition).ctfpickup=allstats(idposition).ctfpickup+filestats(counter2).ctfpickup
				allstats(idposition).ctfcapture=allstats(idposition).ctfcapture+filestats(counter2).ctfcapture
				allstats(idposition).ctfreturn=allstats(idposition).ctfreturn+filestats(counter2).ctfreturn
				'We don't know, in which order the operating system will give us the log files.
				'(They don't necessarily have to be in chronological order.)
				'So we have to check, if the current time stamp is older than the new one and replace it accordingly
				if allstats(idposition).lasttimeactive < filestats(counter2).lasttimeactive then
					allstats(idposition).lasttimeactive=filestats(counter2).lasttimeactive
				end if
			next counter2
		else
			'This never was a Nexuiz stats file, to begin with, so just close it, and continue with the next file
			close 1
		end if
	end if
next counter
'OK, all files and tables aggregated, now let's calculate some values
if makestats=1 then
	if newfilescounter = 0 then
		print "No new files found for integration."
	else
		print newfilescounter;" new files found and integrated."
	end if
end if

'Get the actual current time for calculation purposes
currenttime=datevalue(date)


for counter = 1 to ubound(allstats)
	allstats(counter).alldeaths=allstats(counter).deaths+allstats(counter).Accidents+allstats(counter).suicides
	if allstats(counter).alldeaths+allstats(counter).kills <> 0 then
		allstats(counter).efficiency=allstats(counter).kills/(allstats(counter).alldeaths+allstats(counter).kills)
	else
		allstats(counter).efficiency=0
	end if
	if allstats(counter).alldeaths <> 0 then
		allstats(counter).ratio=allstats(counter).kills/allstats(counter).alldeaths
	else
		allstats(counter).ratio=allstats(counter).kills
	end if
	if allstats(counter).ctfcapture <> 0 then
		allstats(counter).ctfratio=allstats(counter).ctfcapture/allstats(counter).ctfsteal
	else
		allstats(counter).ctfratio=allstats(counter).ctfcapture
	end if
	'We only want the difference between today and last action
	allstats(counter).lasttimeactive=currenttime-allstats(counter).lasttimeactive
next counter

'let's sort by ratio:

changed1=numberofentriesintable(allstats())
while changed1>1
changed2=1
	for counter = 1 to changed1-1
		if (allstats(counter).ratio < allstats(counter + 1).ratio) then
			swap allstats(counter).nick,allstats(counter+1).nick
			swap allstats(counter).kills,allstats(counter+1).kills
			swap allstats(counter).deaths,allstats(counter+1).deaths
			swap allstats(counter).Suicides,allstats(counter+1).Suicides
			swap allstats(counter).Accidents,allstats(counter+1).Accidents
			swap allstats(counter).teamkills,allstats(counter+1).teamkills
			swap allstats(counter).isbot,allstats(counter+1).isbot
			swap allstats(counter).ratio,allstats(counter+1).ratio
			swap allstats(counter).efficiency,allstats(counter+1).efficiency
			swap allstats(counter).alldeaths,allstats(counter+1).alldeaths
			swap allstats(counter).allratio,allstats(counter+1).allratio
			swap allstats(counter).allefficency,allstats(counter+1).allefficency
			swap allstats(counter).ctfsteal,allstats(counter+1).ctfsteal
			swap allstats(counter).ctfdrop,allstats(counter+1).ctfdrop
			swap allstats(counter).ctfpickup,allstats(counter+1).ctfpickup
			swap allstats(counter).ctfcapture,allstats(counter+1).ctfcapture
			swap allstats(counter).ctfreturn,allstats(counter+1).ctfreturn
			swap allstats(counter).ctfratio,allstats(counter+1).ctfratio
			swap allstats(counter).lasttimeactive,allstats(counter+1).lasttimeactive
			changed2=counter
		end if
	next counter
	changed1=changed2
wend



'############## Now let's export our findings ##############



'Changing back is for pathname-compliance with possibly by the user chosen relative paths

chdir startpath

'Regardless of the outputformat and ctf-including or not: if requested, save *all* the stats to a save file
if makestats = 1 then savestats(savefile, filelist(), allstats(), currenttime)

if 0 = open(outfile for output as #2) then
	select case outputfileformat
		case 0
			print #2,"<table>"
			print #2,"<tr>"
			if ctf=0 then
				print #2,"<th>Plc</th><th>Nick</th><th>Kills</th><th>Frgd</th><th>Acc</th><th>Suic</th><th>Deaths</th><th>TKs</th><th>Rat.</th><th>Eff.</th><th>LTS</th>"
			else
				print #2,"<th>Plc</th><th>Nick</th><th>Kills</th><th>Frgd</th><th>Acc</th><th>Suic</th><th>Deaths</th><th>TKs</th><th>Rat.</th><th>Eff.</th><th>Steals</th><th>Drps</th><th>Pick</th><th>Capt.</th><th>Ret.</th><th>CTF-rat.</th><th>LTS</th>"
			end if
			print #2,"</tr>"
			counter=1
			counter2=0
			for counter = 1 to numberofentriesintable(allstats())
				if not (allstats(counter).isbot > 0 and nobots=1) and (allstats(counter).lasttimeactive < datefilter or datefilter=0) then
					counter2=counter2+1
					print #2,"<tr>"
					print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
					print #2,counter2
					print #2,"</td>"
					print #2,"<td>"
					/'Including some cosmetics for the colors and some
					precautions, to avoid html-exploits by deliberately
					malformed nicknames. This should /not/ to be done
					earlier, apllied on the original string, as the
					nicks should be exportable as an unchanged string as
					well, e. g. saved state file or text output file'/
					print #2,nickcolored(htmlonlytext(allstats(counter).nick))
					print #2,"</td>"
					print #2,"<td>"
					print #2,allstats(counter).kills
					print #2,"</td>"
					print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
					print #2,allstats(counter).deaths
					print #2,"</td>"
					print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
					print #2,allstats(counter).Accidents
					print #2,"</td>"
					print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
					print #2,allstats(counter).suicides
					print #2,"</td>"
					print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
					print #2,allstats(counter).alldeaths
					print #2,"</td>"
					print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
					print #2,allstats(counter).teamkills
					print #2,"</td>"
					print #2,"<td style=";chr(34);"text-align: center;";chr(34);">"
					print #2,trailzeros(marginless(str(allstats(counter).ratio)),2)
					print #2,"</td>"
					print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
					print #2,round(allstats(counter).efficiency*100,0,1);"%"
					print #2,"</td>"
					'Only export this data, when being told to do so by the user.
					if ctf=1 then
						print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
						print #2,allstats(counter).ctfsteal
						print #2,"</td>"
						print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
						print #2,allstats(counter).ctfdrop
						print #2,"</td>"
						print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
						print #2,allstats(counter).ctfpickup
						print #2,"</td>"
						print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
						print #2,allstats(counter).ctfcapture
						print #2,"</td>"
						print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
						print #2,allstats(counter).ctfreturn
						print #2,"</td>"
						print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
						print #2,round(allstats(counter).ctfratio*100,0,1);"%"
						print #2,"</td>"
					end if
					print #2,"<td style=";chr(34);"text-align: right;";chr(34);">"
					print #2,allstats(counter).lasttimeactive;" d"
					print #2,"</td>"
						
					print #2,"</tr>"
				else
				end if
			next counter
			print #2,"</table>"
		case 1
			/'This time with the nick at the last position, as you won't
			have to worry about spaces in nicks, then. Everything after
			the first "% " in that line /must/ be part of the nick.
			Makes it easier to work with, later. Also no color-stuff or
			other manipulation shall be used here. This is mostly meant
			for use with other programs, so let's leave any changes to
			them.'/
			print #2,"Place Kills Fragged Accidents Suicides Deaths TKs Ratio Eff. Steals Drops Pickups Captures Returns CTF-ratio Last-time-seen Nick"
			counter2=0
			for counter = 1 to numberofentriesintable(allstats())
				if not (allstats(counter).isbot > 0 and nobots=1) and (allstats(counter).lasttimeactive < datefilter or datefilter=0) then
					counter2=counter2+1
					print #2,counter2;" ";
					print #2,allstats(counter).kills;" ";
					print #2,allstats(counter).deaths;" ";
					print #2,allstats(counter).Accidents;" ";
					print #2,allstats(counter).suicides;" ";
					print #2,allstats(counter).alldeaths;" ";
					print #2,allstats(counter).teamkills;" ";
					print #2,trailzeros(marginless(str(allstats(counter).ratio)),2);" ";
					print #2,round(allstats(counter).efficiency*100,0,1);"% ";
					print #2,allstats(counter).ctfsteal;" ";
					print #2,allstats(counter).ctfdrop;" ";
					print #2,allstats(counter).ctfpickup;" ";
					print #2,allstats(counter).ctfcapture;" ";
					print #2,allstats(counter).ctfreturn;" ";
					print #2,round(allstats(counter).ctfratio*100,0,1);"% ";
					print #2,allstats(counter).lasttimeactive;" ";
					print #2,allstats(counter).nick;" "
				end if
			next counter
		case 2
			/'This is meant to be compatible with usual spread sheet
			software as LibreOffice or Microsoft Excel. It therefore has
			to fulfill certain standards, but nicks don't need to be
			changed more extensively.'/
			print #2,"Place,Kills,Fragged,Accidents,Suicides,Deaths,TKs,Ratio,Eff.,Steals,Drops,Pickups,Captures,Returns,CTF-ratio,Last-time-seen,Nick"
			counter2=0
			for counter = 1 to numberofentriesintable(allstats())
				if not (allstats(counter).isbot > 0 and nobots=1) and (allstats(counter).lasttimeactive < datefilter or datefilter=0) then
					counter2=counter2+1
					print #2,counter2;",";
					print #2,allstats(counter).kills;",";
					print #2,allstats(counter).deaths;",";
					print #2,allstats(counter).Accidents;",";
					print #2,allstats(counter).suicides;",";
					print #2,allstats(counter).alldeaths;",";
					print #2,allstats(counter).teamkills;",";
					print #2,trailzeros(marginless(str(allstats(counter).ratio)),2);",";
					print #2,csvstring(round(allstats(counter).efficiency*100,0,1)+"%");",";
					print #2,allstats(counter).ctfsteal;",";
					print #2,allstats(counter).ctfdrop;",";
					print #2,allstats(counter).ctfpickup;",";
					print #2,allstats(counter).ctfcapture;",";
					print #2,allstats(counter).ctfreturn;",";
					print #2,csvstring(round(allstats(counter).ctfratio*100,0,1)+"%");",";
					print #2,csvstring(str(allstats(counter).lasttimeactive)+" d");",";
					print #2,csvstring(allstats(counter).nick)
				end if
			next counter
		case 3
			/'Probably the most demanding, if it comes to the nicks.
			It's intended to be conforming with standard (being "well
			formed"), so there aren't any characters allowed in here,
			that are not part of the ISO 8859-1 character set. So they
			are simply removed from the nicks. At this point, one might
			implement a Unicode conversion and make it be a XML file
			using UTF-8.'/
			print #2,"<?xml version=";chr(34);"1.0";chr(34);" encoding=";chr(34);"ISO-8859-1";chr(34);" standalone=";chr(34);"yes";chr(34);"?>"
			print #2,"<nexuizstatsexport>"
			print #2,"<heading>"
			if ctf=0 then
				print #2,"<placeheading>Plc</placeheading>"
				print #2,"<nickheading>Nick</nickheading>"
				print #2,"<killsheading>Kills</killsheading>"
				print #2,"<fraggedheading>Frgd</fraggedheading>"
				print #2,"<accuracyheading>Acc</accuracyheading>"
				print #2,"<suicideheading>Suic</suicideheading>"
				print #2,"<deathsheading>Deaths</deathsheading>"
				print #2,"<teamkillsheading>TKs</teamkillsheading>"
				print #2,"<ratioheading>Rat.</ratioheading>"
				print #2,"<efficiencyheading>Eff.</efficiencyheading>"
				print #2,"<lasttimeseenheading>LTS</lasttimeseenheading>"
			else
				print #2,"<placeheading>Plc</placeheading>"
				print #2,"<nickheading>Nick</nickheading>"
				print #2,"<killsheading>Kills</killsheading>"
				print #2,"<fraggedheading>Frgd</fraggedheading>"
				print #2,"<accuracyheading>Acc</accuracyheading>"
				print #2,"<suicideheading>Suic</suicideheading>"
				print #2,"<deathsheading>Deaths</deathsheading>"
				print #2,"<teamkillsheading>TKs</teamkillsheading>"
				print #2,"<ratioheading>Rat.</ratioheading>"
				print #2,"<efficiencyheading>Eff.</efficiencyheading>"
				print #2,"<stealsheading>Steals</stealsheading>"
				print #2,"<dropsheading>Drps</dropsheading>"
				print #2,"<pickupsheading>Pick</pickupsheading>"
				print #2,"<capturesheading>Capt.</capturesheading>"
				print #2,"<returnsheading>Ret.</returnsheading>"
				print #2,"<ctfratioheading>CTF-rat.</ctfratioheading>"
				print #2,"<lasttimeseenheading>LTS</lasttimeseenheading>"
			end if
			print #2,"</heading>"
			print #2,"<playerentries>"
			counter=1
			counter2=0
			for counter = 1 to numberofentriesintable(allstats())
				if not (allstats(counter).isbot > 0 and nobots=1) and (allstats(counter).lasttimeactive < datefilter or datefilter=0) then
					counter2=counter2+1
					print #2,"<playerentry>"
					print #2,"<place>"
					print #2,counter2
					print #2,"</place>"
					print #2,"<nickname>"
					'Including some precautions, to make it well formed
					print #2,xmlonlytext(allstats(counter).nick)
					print #2,"</nickname>"
					print #2,"<kills>"
					print #2,allstats(counter).kills
					print #2,"</kills>"
					print #2,"<deaths>"
					print #2,allstats(counter).deaths
					print #2,"</deaths>"
					print #2,"<accidents>"
					print #2,allstats(counter).Accidents
					print #2,"</accidents>"
					print #2,"<suicides>"
					print #2,allstats(counter).suicides
					print #2,"</suicides>"
					print #2,"<alldeaths>"
					print #2,allstats(counter).alldeaths
					print #2,"</alldeaths>"
					print #2,"<teamkills>"
					print #2,allstats(counter).teamkills
					print #2,"</teamkills>"
					print #2,"<ratio>"
					print #2,trailzeros(marginless(str(allstats(counter).ratio)),2)
					print #2,"</ratio>"
					print #2,"<efficiency>"
					print #2,round(allstats(counter).efficiency*100,0,1);"%"
					print #2,"</efficiency>"
					if ctf=1 then
						print #2,"<steals>"
						print #2,allstats(counter).ctfsteal
						print #2,"</steals>"
						print #2,"<drops>"
						print #2,allstats(counter).ctfdrop
						print #2,"</drops>"
						print #2,"<pickups>"
						print #2,allstats(counter).ctfpickup
						print #2,"</pickups>"
						print #2,"<captures>"
						print #2,allstats(counter).ctfcapture
						print #2,"</captures>"
						print #2,"<returns>"
						print #2,allstats(counter).ctfreturn
						print #2,"</returns>"
						print #2,"<ctfratio>"
						print #2,round(allstats(counter).ctfratio*100,0,1);"%"
						print #2,"</ctfratio>"
					end if
					print #2,"<lasttimeactive>"
					print #2,allstats(counter).lasttimeactive;" d"
					print #2,"</lasttimeactive>"
					print #2,"</playerentry>"
				else
				end if
			next counter
			print #2,"</playerentries>"
			print #2,"</nexuizstatsexport>"
		case else
			'shouldn't be happening
	end select
	close 2
else
	print "Couldn't create file ";chr(34);outfile;chr(34);", aborting!"
	errorflag=6
	system(errorflag)
end if

